#!/usr/bin/perl
use strict;
use LWP::UserAgent;
use Data::Dumper;

my $link = $ARGV[0];

if(! $link ) {
  $link = "http://ecsinfo.gsfc.nasa.gov/ECSInfo/ecsmetadata/METADATA/Valids_doc/drop6a0720030423valids_gsfc_ops.txt";
}

my $baseurl = $link;
#print "BASEURL: $baseurl\n" unless ($baseurl);

my %validshash = ();
my @lines=();

open COLLOUTFILE, ">./collection_valids.xsd"
	or die "couldn't open for appending: $!\n";
open GRANOUTFILE, ">./granule_valids.xsd"
	or die "couldn't open for appending: $!\n";
open ORIGFILE, ">./valids.dat"
	or die "couldn't open for appending: $!\n";
open LOGFILE, ">>./logs/v2s.log"
	or die "couldn't open for appending: $!\n";
my $logCounter=0;

my $res = '';

my $ua = new LWP::UserAgent();     

getcontent ($link);
process(@lines);

close LOGFILE;
close COLLOUTFILE;
close GRANOUTFILE;
close ORIGFILE;

exit 0;

sub getcontent {
    my ($url) = @_;

    my $linkarr = '';
    my $res = $ua->request (new HTTP::Request ('GET' => $url));

    if ($res->is_success) {
    print LOGFILE $logCounter . "- Got content from $link...\n";
    $logCounter++;
    @lines = split /\n/, $res->as_string;
    print ORIGFILE $res->as_string;
    } else {
    print LOGFILE $logCounter . "- Failed to get content from $link...\n";
    $logCounter++;
    }

}

sub process{
   my (@lines) = @_;
   my $op    = "";
   my $min   = "";
   my $max   = "";
   my $enum  = "";
   my $collection_flag=1;
   my $i=0;
 print COLLOUTFILE <<INIT;
<?xml version="1.0" encoding="UTF-8"?>
<xs:schema targetNamespace="http://shiraz.gsfc.nasa.gov/Schema/bgmt" xmlns:bmgt="http://shiraz.gsfc.nasa.gov/Schema/bgmt" xmlns:xs="http://www.w3.org/2001/XMLSchema" elementFormDefault="qualified" attributeFormDefault="unqualified">
INIT
   for ($i=14; $i<@lines;$i++) {
	   my $line = clean(@lines[$i]);
	   if($line =~ m/INVENTORYMETADATA Attribute Valids/i) {
		   print GRANOUTFILE <<INIT;
<?xml version="1.0" encoding="UTF-8"?>
<xs:schema targetNamespace="http://shiraz.gsfc.nasa.gov/Schema/bgmt" xmlns:bmgt="http://shiraz.gsfc.nasa.gov/Schema/bgmt" xmlns:xs="http://www.w3.org/2001/XMLSchema" elementFormDefault="qualified" attributeFormDefault="unqualified">
INIT
		$collection_flag=0;
		next;
	   } elsif ($line =~ m/(\w+):$/) {
		if($collection_flag) {
		print COLLOUTFILE "<xs:simpleType name=\"$1Type\" final=\"restriction\">\n";
		} else {
		print GRANOUTFILE "<xs:simpleType name=\"$1Type\" final=\"restriction\">\n";
		}
		my $lookahead = @lines[$i+1];
		if (! ($lookahead =~ m/Expression\(/ || 
                       $lookahead =~ m/Range\(/) ){
		if($collection_flag) {
		print COLLOUTFILE "\t<xs:restriction base=\"xs:string\">\n";
		} else {
		print GRANOUTFILE "\t<xs:restriction base=\"xs:string\">\n";
		}
		}
	   } elsif ($line =~ m/Expression\((.*)\)/ ) {
		my $exp = $1;
		($op,$min) = split('\,', $exp); 
		if ($min =~ m/\./) {
		if($collection_flag) {
	  	   print COLLOUTFILE "\t<xs:restriction base=\"xs:float\">\n";
		} else {
	  	   print GRANOUTFILE "\t<xs:restriction base=\"xs:float\">\n";
			}
		} else {
		if($collection_flag) {
		   print COLLOUTFILE "\t<xs:restriction base=\"xs:integer\">\n";
		 } else {
		   print GRANOUTFILE "\t<xs:restriction base=\"xs:integer\">\n";
		 }
		}
		if($op eq ">") {
		if($collection_flag) {
	  	   print COLLOUTFILE "\t\t<xs:minExclusive value=\"$min\"/>\n";
		 } else {
	  	   print GRANOUTFILE "\t\t<xs:minExclusive value=\"$min\"/>\n";
			}
		} elsif ($op eq ">=") {
		if($collection_flag) {
	  	   print COLLOUTFILE "\t\t<xs:minInclusive value=\"$min\"/>\n";
			} else {
	  	   print GRANOUTFILE "\t\t<xs:minInclusive value=\"$min\"/>\n";
			}
		}
	   } elsif ($line =~ m/Range\((.*)\)/ ) {
		my $range = $1;
		($min,$max) = split ('\,',$range); 
		if ($min =~ m/\./ || $max =~ m/\./) {
		if($collection_flag) {
	  	   print COLLOUTFILE "\t<xs:restriction base=\"xs:float\">\n";
			} else {
	  	   print GRANOUTFILE "\t<xs:restriction base=\"xs:float\">\n";
			}
		} else {
		if($collection_flag) {
		   print COLLOUTFILE "\t<xs:restriction base=\"xs:integer\">\n";
			} else {
		   print GRANOUTFILE "\t<xs:restriction base=\"xs:integer\">\n";
			}
		}
		if($collection_flag) {
	  	   print COLLOUTFILE "\t\t<xs:minInclusive value=\"$min\"/>\n";
	  	   print COLLOUTFILE "\t\t<xs:maxInclusive value=\"$max\"/>\n";
		 } else {
	  	   print GRANOUTFILE "\t\t<xs:minInclusive value=\"$min\"/>\n";
	  	   print GRANOUTFILE "\t\t<xs:maxInclusive value=\"$max\"/>\n";
		 }
	   } elsif( $line =~ /\t/ ){
		$enum = strip($line);
		if($collection_flag) {
		print COLLOUTFILE "\t\t<xs:enumeration value=\"$enum\"/>\n";
		} else {
		print GRANOUTFILE "\t\t<xs:enumeration value=\"$enum\"/>\n";
		}
	   } elsif( $line eq "") {
		if (@lines[$i-1] ne "" && 
                    !(@lines[$i-1] =~ m/INVENTORY|COLLECTION/)) {
		if($collection_flag) {
		print COLLOUTFILE 	"\t</xs:restriction>\n";
		print COLLOUTFILE   "</xs:simpleType>\n";
		} else {
		print GRANOUTFILE 	"\t</xs:restriction>\n";
		print GRANOUTFILE   "</xs:simpleType>\n";
		}
		}

	   }



	
   }
   print COLLOUTFILE   "</xs:schema>\n";
   print GRANOUTFILE 	"\t</xs:restriction>\n";
   print GRANOUTFILE   "</xs:simpleType>\n";
   print GRANOUTFILE   "</xs:schema>\n";


}

sub clean{
	my($str) = @_;
	$str =~ s/&/&amp;/g;
	return $str;
}

sub strip{
   my ($str) = @_;
	$str =~ s/^\t//g;
	return $str;
}
1;
